@@ -18,21 +18,19 @@
 import imp, time, threading, Context, rightsManager, os, zipimport, sys
 from decorators import *
 
-class Logic:
+class Core:
 	def __init__(self, bot, config):
 		sys.path.append(os.getcwd() + "/logics")
+		self.config = config
 		self.rights = rightsManager.RightsManager(config.rights)
 		self.bot = bot
 		self.logics = {}
 		self.registers = {}
-		self.events = {"joinevent": {}, "kickevent": {}, "leftevent": {}, "renameevent": {}, "modeevent": {}, "numanswer": {}}
+		self.events = {"joinevent": {}, "kickevent": {}, "leftevent": {}, "renameevent": {}, "modeevent": {}, "numanswer": {}, "botjoinevent": {}}
 		self.globalCalls = {}
 		self.commands = {}
 		self.bot.threads = {}
 		self.open_logic("admin")
-		self.grep = ""
-		self.noticing = False
-		self.timing = False
 		self.last_arg = ""
 		for module in config.logics_startup:
 			self.open_logic(module)
@@ -119,37 +117,41 @@ class Logic:
 		else:
 			return False
 	
-	def exec_commande (self, user, channel, message, command, command_args):
+	def exec_commande (self, ctxte, command):
+		command = command.strip()
 		if command == "":
 			return
 		if self.commands.has_key(command):
 			logic = self.logics[self.commands[command][0]]
 			cmd_func = getattr(logic, self.commands[command][1])
-			if hasattr(cmd_func, "is_admin") and not self.has_exec_right(user, self.commands[command][0], self.commands[command][1]) :
-				self.bot.say(channel, "The command %s is an admin command" % self.commands[command][1])
+			admin = self.has_exec_right(ctxte.user, self.commands[command][0], self.commands[command][1])
+			if hasattr(cmd_func, "is_admin") and not admin :
+				ctxte.say("The command %s is an admin command" % self.commands[command][1])
 				return
-			ctxte = Context.Context(cmd_func, user, channel, message, self.bot, command_args)
-			ctxte.grep = self.grep
-			ctxte.noticing = self.noticing
-			ctxte.timing = self.timing
+			ctxte.cmd_func = cmd_func
+			ctxte.is_admin = admin
 			ctxte.launch()
 		else:
-			self.bot.say(channel, "Command %s not found" % command)
+			ctxte.say("Command %s not found" % command)
 	
-	def answer_internal (self, user, channel, message):
-		print (user, channel, message)
+	
+	# Hooks return "" for an error, nothing when the hook is not concerned, the message without the hook when it applies.
+	def hook_time (self, ctxte, message):
 		if message.startswith("time "):
-			self.timing = True
-			message = message[4:].strip()
-			self.answer_internal(user, channel, message)
-		elif message.startswith("notice ") and not self.bot.noticing:
-			self.noticing = True
-			message = message[6:].strip()
-			self.answer_internal(user, user, message)
-		elif message.startswith("grep "):
+			ctxte.timing = True
+			return message[4:].strip()
+		
+	def hook_notice (self, ctxte, message):
+		if message.startswith("notice "):
+			ctxte.noticing = True
+			return message[6:].strip()
+		
+	def hook_grep (self, ctxte, message):
+		if message.startswith("grep "):
 			message = message[5:].strip()
 			if not message.startswith('"'):
-				self.bot.say(channel, 'Usage : grep "txt" command')
+				ctxte.say('Usage : grep "txt" command')
+				return ""
 			else:
 				i = 1
 				while i < len(message):
@@ -157,44 +159,56 @@ class Logic:
 						break
 					i += 1
 				if i == len(message):
-					self.bot.say(channel, 'Usage : grep "txt" command')
+					ctxte.say('Usage : grep "txt" command')
+					return ""
 				else:
 					criteria = message[1:i]
 					command = message[i+1:].strip()
 					if len(command) == 0:
-						self.bot.say(channel, 'Usage : grep "txt" command')
+						ctxte.say('Usage : grep "txt" command')
 					else:
-						self.grep = criteria
-						self.answer_internal(user, channel, command)
-		else:
+						ctxte.grep = criteria
+						return command
+
+	def answer(self, ctxte):
+		for call in self.globalCalls.values():
+			call(ctxte)
+		if ctxte.message.startswith(self.bot.nickname) and not ctxte.stop:
+			# Set the right message value now
+			ctxte.message = ctxte.message[len(self.bot.nickname) + 1:].strip()
+			message = ctxte.message
+			hooks = ["time", "notice", "grep"]
+			while 1:
+				tocontinue = False
+				for hook in hooks:
+					result = getattr(self, "hook_" + hook)(ctxte, message)
+					if result == "":
+						return
+					elif result != None:
+						tocontinue = True
+						hooks.remove(hook)
+						message = result
+						ctxte.message = message
+				if not tocontinue or len(hooks) == 0:
+					break
+			while "&&" in message:
+					message = message.replace("&&", "&")
+			while " & " in message:
+				message = message.replace(" & ", "&")
 			if " " in message:
-				[command, command_args] = message.split(" ",1)
-				if command_args in ["*", "!*"]:
-					command_args = self.last_arg
+				[command, ctxte.command_args] = message.split(" ",1)
+				if ctxte.command_args in ["*", "!*"]:
+					ctxte.command_args = self.last_arg
 				else:
-					self.last_arg = command_args
+					self.last_arg = ctxte.command_args
 			else:
 				command = message
-				command_args = ""
+				ctxte.command_args = ""
 			if "&" in command:
-				while " & " in message:
-					message = message.replace(" & ", "&")
-				while "&&" in command:
-					command = command.replace("&&", "&")
 				commands = command.split("&")
 				for command in commands:
-					self.exec_commande(user, channel, message, command.strip(), command_args)
+					print "executing", command
+					self.exec_commande(ctxte, command)
 			else:
-				self.exec_commande(user, channel, message, command, command_args)
-			self.grep = ""
-			self.noticing = False
-			self.timing = False
-
-	def answer(self, user, channel, message):
-		user = user.split('!', 1)[0]
-		for call in self.globalCalls.values():
-			call(user, channel, message)
-		if message.startswith(self.bot.nickname):
-			# Remove the bot name
-			message = message[len(self.bot.nickname) + 1:].strip()
-			self.answer_internal(user,channel,message)
+				print "executing", command
+				self.exec_commande(ctxte, command)
