@@ -18,36 +18,37 @@
 
 
 from IRCClient import IRCClient
-import imp, IRC_conf, time, inspect
-
-config = IRC_conf.IRCConf("bot_conf.xml")
+import imp, IRC_conf, time, Context
 
 class ModularBot(IRCClient):
-	def __init__(self, host, port, nick, ident, realname):
-		IRCClient.__init__(self, host, port, nick, ident, realname)
+	def __init__(self, config):
+		self.config = config
+		IRCClient.__init__(self, config.url, config.port, config.nick, config.ident, config.realname)
 		self.noticing = False
-		self.grep = ""
 		self.nickname = config.nick
 		for attr in ["versionNum", "versionName", "versionEnv"]:
 			setattr(self, attr, getattr(config, attr))
 		self.core_mod = imp.load_source("", "Core.py")
-		self.logic = self.core_mod.Logic(self, config)
+		self.core = self.core_mod.Core(self, self.config)
 		self.versionString = "%s@%s - %s" % (self.versionName, self.versionEnv, self.versionNum)
 
-	def privmsg(self, user, channel, msg):
-		if channel.lower() == self.nickname.lower():
-			msg = self.nickname + ": " + msg
-			channel = user.split("!", 1)[0]
-		if msg == "%s: CORE_RELOAD" % self.nickname:
-			if self.logic.rights.is_core_admin(user):
+	def privmsg(self, ctxte):
+		if ctxte.message == "%s: CORE_RELOAD" % self.nickname:
+			if self.core.rights.is_core_admin(user) and self.core.rights.logged_in(user):
 				print "------CORE RELOAD------"
+				del self.core
 				self.core_mod = imp.load_source("", "Core.py")
-				self.logic = self.core_mod.Logic(self, config)
+				self.core = self.core_mod.Core(self, self.config)
 				return
-		self.logic.answer(user, channel, msg)
+		self.core.answer(ctxte)
 	
 	def say (self, channel, message):
-		#print "Appel venant de ", inspect.stack()[1][3]
+		try:
+			msg = message.encode("utf-8")
+			message = msg
+		except:
+			print "Encoding problem with %s" % message
+		#print "Call from ", inspect.stack()[1][3]
 		if len(message) > 450:
 			broke = False
 			part1 = ""
@@ -87,20 +88,49 @@ class ModularBot(IRCClient):
 	def joined(self, channel):
 		for logon in config.logons:
 			eval(logon)
+		self.core.event("botjoinevent", channel, [])
 	def userJoined(self, user, channel):
-		self.logic.event("joinevent", channel, [user, channel])
+		self.core.event("joinevent", channel, [user, channel])
 	def userKicked(self, user, channel, kicker, reason):
-		self.logic.event("kickevent", channel, [user, channel, kicker, reason])
+		self.core.event("kickevent", channel, [user, channel, kicker, reason])
 	def userRenamed(self, oldname, newname):
-		self.logic.event("renameevent", "", [oldname, newname])
+		self.core.event("renameevent", "", [oldname, newname])
 	def userLeft(self, user, channel):
-		self.logic.event("leftevent", channel, [user, channel])
+		self.core.event("leftevent", channel, [user, channel])
 	def modeChanged (self, user, channel, set, modes, args):
-		self.logic.event("modeevent", channel, [user, channel, set, modes, args])
+		self.core.event("modeevent", channel, [user, channel, set, modes, args])
 	def numericAnswer(self, cmd_number, line):
-		self.logic.event("numanswer", "", [cmd_number, line])
+		self.core.event("numanswer", "", [cmd_number, line])
+	
+	def PRIVMSG (self, line, line_splitted):
+		channel = line_splitted[2]
+		user = line_splitted[0][1:]
+		msg = line[len(user) + len(channel) + len(line_splitted[1]) + 5:]
+		if msg[0] == chr(1) and msg[-1] == chr(1):
+			ctcp_cmd = msg[1:-1].split(" ", 1)[0]
+			if hasattr(self, "CTCP_" + ctcp_cmd):
+				getattr(self, "CTCP_" + ctcp_cmd)(user, msg)
+			else:
+				print "CTCP command %s not found" % ctcp_cmd
+		else:
+			ctxte = Context.Context(user, channel, self, msg)
+			self.privmsg(ctxte)
+	
+	def NOTICE (self, line, line_splitted):
+		print "Notice not handled currently (risk of infinite loop)"
+		channel = line_splitted[2]
+		user = line_splitted[0][1:]
+		print (channel, user)
+		return
+		channel = line_splitted[2]
+		user = line_splitted[0][1:]
+		msg = line[len(user) + len(channel) + len(line_splitted[1]) + 5:]
+		ctxte = Context.Context(user, channel, self, msg)
+		ctxte.from_notice = True
+		self.privmsg(ctxte)
 
 if __name__ == "__main__":
-	client = ModularBot (config.url, config.port, config.nick, config.ident, config.realname)
+	config = IRC_conf.IRCConf("bot_conf.xml")
+	client = ModularBot (config)
 	client.connect()
 	client.run()
\ No newline at end of file
