@@ -47,6 +47,7 @@ class IRCClient:
 	def __init__ (self, host, port, nick, ident, realname):
 		for var in ["host", "port", "nick", "ident", "realname"]:
 			setattr(self, var, locals()[var])
+		self.nickname = nick
 		self.sock = socket.socket()
 		self.versionString = "modularbot"
 		self.floodmgr = FloodManager()
@@ -72,7 +73,8 @@ class IRCClient:
 		if not self.sock:
 			print "Could not open socket"
 			sys.exit(1)
-		self.IRC_send("NICK %s" % self.nick)
+		self.sock.settimeout(90)
+		self.IRC_send("NICK %s" % self.nickname)
 		self.IRC_send("USER %s %s bla :%s" % (self.ident, self.host, self.realname))
 	
 	def join (self, channel):
@@ -162,32 +164,42 @@ class IRCClient:
 	
 	def run (self):
 		readbuffer = ""
+		sent_ping = False
 		while 1:
-			readbuffer = readbuffer + self.sock.recv (1024)
 			try:
-				temp = readbuffer.split ("\n")
-				readbuffer = temp.pop ()
-				for line in temp:
-					line = line.rstrip()
-					print line
-					line_splitted = line.split()
-					if line_splitted[0] == "PING":
-						self.IRC_send ("PONG %s" % line_splitted[1])
-					else:
-						if hasattr(self, line_splitted[1]):
-							getattr(self, line_splitted[1])(line, line_splitted)
+				readbuffer = readbuffer + self.sock.recv (1024)
+				try:
+					temp = readbuffer.split ("\n")
+					readbuffer = temp.pop ()
+					for line in temp:
+						line = line.rstrip()
+						print line
+						line_splitted = line.split()
+						if line_splitted[0] == "PING":
+							self.IRC_send ("PONG %s" % line_splitted[1])
+						elif line_splitted[1] == "PONG":
+							sent_ping = False
 						else:
-							try:
-								cmd_number = int(line_splitted[1])
-								if cmd_number == 376:
+							if hasattr(self, line_splitted[1]):
+								getattr(self, line_splitted[1])(line, line_splitted)
+							else:
+								try:
+									cmd_number = int(line_splitted[1])
+									if cmd_number == 376:
 										self.signedOn()
-								self.numericAnswer(cmd_number, line.split(" ", 3)[-1])
-							except ValueError:
-								pass
-							#print "Unknown IRC command !!!!"
-			except:
-				infos = sys.exc_info()
-				sys.excepthook(infos[0], infos[1], infos[2])
+									self.numericAnswer(cmd_number, line.split(" ", 3)[-1])
+								except ValueError:
+									pass
+								#print "Unknown IRC command !!!!"
+				except:
+					infos = sys.exc_info()
+					sys.excepthook(infos[0], infos[1], infos[2])
+			except socket.timeout:
+				if not sent_ping:
+					self.IRC_send("PING %s;" % self.host)
+					sent_ping = True
+				else:
+					raise
 	
 	#EVENTS :
 	def signedOn(self):
